<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class BarLine extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_BARLINE = "5898c8f3ca6bc3c1f61960d9";
    const TYPE_ECHART = "bar";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $chartSeries = $this->mergeChartSeries($widget['chartSeries']);
        $serieTypeShow = [true, true];
        $maskIndex = ["serieYAxisFormatBar", "serieYAxisFormatLine"];
        $yAxisArray = [];
        $i = 0;
        foreach($chartSeries as $serie){
            $i = $serie["serieType"] == "bar" ? 0 : 1;
            $yAxis = [
                "type" => "value",
                "show" => $serieTypeShow[$i],
                "scale" => true,
                "axisLabel" => [
                    "show" => true,
                ],
            ];
            if (!empty($widget[$maskIndex[$i]])){
                $yAxis["maskFunction"] = $widget[$maskIndex[$i]];
            }
            $serieTypeShow[$i] = false;
            $yAxisArray[] = $yAxis;
        }

        return $yAxisArray;
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            if(!empty($row[$widget['chartXAxisField']]) && !in_array($row[$widget['chartXAxisField']], $categoria)){
                $categoria[] = $row[$widget['chartXAxisField']];
            }
        }
        $rotate = 0;
        if(!empty($widget['chartXAxisTicksRotation']) || (isset($widget['chartXAxisTicksRotation']) && $widget['chartXAxisTicksRotation'] === 0) )
        {
            $rotate = $widget['chartXAxisTicksRotation'];
        }
        $yAxis = [
            "type" => "category",
            "data" => $categoria,
            "axisLabel"=> [
                "rotate" =>  $rotate,
                "interval"=>  0,
                "overflow"=> "truncate",
                "hideOverlap"=> true
            ]
        ];
        if($rotate !== 0) {
            $yAxis["axisLabel"]["width"] = 100;
        }
        return $yAxis;
    }

    protected function mergeChartSeries($chartSeries) {
        $mergedSeries = [];
        foreach($chartSeries as $serie){
            foreach($serie["serieFields"] as $i => $field){
                $newSerie = $serie;
                $newSerie['serieField'] = $serie['serieFields'][$i];
                $newSerie['serieLabel'] = $serie['serieLabels'][$i];
                $newSerie['serieColor'] = $serie['serieColors'][$i];
                unset($newSerie['serieFields'], $newSerie['serieLabels'], $newSerie['serieColors']);
                $mergedSeries[] = $newSerie;
            }
        }
        return $mergedSeries;
    }

    protected function getSeries()
    {
        $widget = $this->widget;
        $colors = $this->getSerieColors();
        $series = [];
        $chartSeries = $this->mergeChartSeries($widget['chartSeries']);
        foreach ($this->widgetData as $row) {
            foreach($chartSeries as $index => $fact){
                if (empty($series[$index])) {
                    $series[$index] = array(
                        "name" => $fact["serieLabel"],
                        "type" => $fact["serieType"],
                        "data" => array(),
                        "label" => [
                            "show" => true,
                            "color"=> $colors[$index],
                            "position" => 'top'
                        ],
                        "yAxisIndex" => $fact["serieType"] == "bar" ? 0 : 1,
                        "stack" => $this->getSerieStacked(),
                    );
                }
                $series[$index]["data"][] = !empty($row[$fact["serieField"]]) ? $row[$fact["serieField"]] : 0;
            }
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $barOptions = [
            "horizontal" => true,
            "dataLabels" => [
                "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top',
                "textAnchor" => "end",
            ],
        ];

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barOptions["barHeight"] = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return [
            "bar" => $barOptions
        ];
    }

    protected function getBarWidth(){
        $widget = $this->widget;
        $barWidth = "auto";

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barWidth = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return $barWidth;
    }

    protected function getBarHeightByDimensionGap($dimensionGap){
        return strval((1-$dimensionGap)*100 )."%";
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "bar",
            "id" => $widget['id'],
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return array_merge($widget["chartSeries"][0]["serieColors"],self::DEFAULT_COLORS);
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["serieShowValueLabels"]) ? $widget["serieShowValueLabels"] : false,
            "position" => "top",
            "textAnchor" => "start",
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "textStyle" => [
                "fontSize" => "16px",
                "fontFamily" => "Arial Black",
                "fontWeight" => "bold",
            ],
            "align" => "left",
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
                "fontSize" => '14px',
            ],
        ];
    }

    protected function getLegend()
    {
        $widget = $this->widget;
        if (!empty($widget["chartLegendPosition"])){
            $string = $widget["chartLegendPosition"];
            $variable = [];
            $variations = explode(" ", $string);
            foreach ($variations as $variation) {
                if ($variation == "top") {
                    $$variable['top'] = "5%";
                    $variable['orient'] = "vertical";
                }
                if ($variation == "bottom") {
                    $variable['bottom'] = "0%";
                    $variable['orient'] = "vertical";
                }
                if ($variation == "left") {
                    $variable['left'] = "left";
                    $variable['orient'] = "vertical";
                }
                if ($variation == "right") {
                    $variable['right'] = "right";
                    $variable['orient'] = "vertical";
                }
            }
            return $variable;
        }
        return [
            "top" => "5%",
            "orient" => "vertical"
        ];
    }

    protected function getDataZoom()
    {
        $data = $this->widgetData;
        $widget = $this->widget;
        $column =  count($data) * count($widget['chartSeries']);
        $dataZoom  = null;
        if($column >= 20) {
            $end = intval(2000/$column);
            if($end < 1) {
                $end = 1;
            } else if($end > 100) {
                $end = 100;
            }
            $dataZoom = [
                [
                    "show" => true,
                    "height" => 10,
                    "end" => $end,
                    "moveHandleSize" => 0,
                    "bottom" => "0%"
                ]
            ];
        }
        return $dataZoom;
    }

    protected function getOptions(){
        $widget = $this->widget;

        $options = [
            "grid" => [
                "left" => '10px',
                "right" => '10px',
                "bottom" => '20px',
                "containLabel" => true
            ],
            "color" => $this->getSerieColors(),
            "legend" => $this->getLegend(),
            "title" => $this->getTitle(),
            "tooltip" => [
                "trigger" => "axis",
                "axisPointer" => [
                    "type" => "shadow"
                ]
            ],
            "xAxis" => $this->getXAxis(),
            "yAxis" => $this->getYAxis(),
            'series' => $this->getSeries(),
            "dataZoom" => $this->getDataZoom()
        ];

        if(empty($options["dataZoom"])){
            $options["grid"]["bottom"] = '0px';
        }
        return $options;
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_BARLINE,
            "component" => self::ECHART_COMPONENT,
            "chartType" => $widget['serieType'],
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
