<?php
namespace Service;

use \Service\Manager\Mongo\Manager;
use Util\OpensslCrypt;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\Mongo\Mongo;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Zmart\QueryBuilders\Mongo\QueryBuilder;

class SignUp {
	
	protected $defaultConnections = 'connection';
    protected $mongoManager;
    protected $nameProvider;
	protected $queryBuilder;
    protected $host;
    protected $port;
    protected $options;
	protected $orgOptions;
	
	public function __construct(Manager $mongoManager, NameProvider $nameProvider, QueryBuilder $queryBuilder, 
								$host, $port, $options, $orgOptions) {
		$this->mongoManager = $mongoManager;
		$this->nameProvider = $nameProvider;
        $this->queryBuilder = $queryBuilder;
		$this->host = $host;
		$this->port = $port;
		$this->options = $options;
		$this->orgOptions = $orgOptions;
	}
	
	public function addNewOrgConnection($connection){
		$dataSet = new DataSet('connection', $connection);
        $this->mongoManager->persist($dataSet);
	}
	
	public function createDbMongo($organizationId, $isOrgOptions = false){
		$options = $isOrgOptions ? $this->orgOptions : $this->options;
		$dbName = 'Zmart__'.$organizationId;
		$orgMongo = new Mongo($this->host, $this->port, $dbName,$options);
		$orgManager = new Manager($orgMongo, $this->nameProvider, $this->queryBuilder);
		try{
            $orgManager->createDbUser($dbName);
        }catch (\Exception $e){
            echo $e->getMessage()."\n";
        }
    }

    public function createConnectionWallet($organizationId, $isSaas, $walletName){
        $dbName = 'Zmart__'.$organizationId;
        $orgMongo = new Mongo($this->host, $this->port, $dbName, $this->orgOptions );
        $orgManager = new Manager($orgMongo, $this->nameProvider, $this->queryBuilder);

        $widgetFilterCriteria = new FilterCriteria('connection', array(
            array(
                "columnName"  => "name",
                "value"       => "MAIN_CONNECTION",
                "operator"    => "="
            )
        ));
        $connection = $orgManager->findBy($widgetFilterCriteria)->getRows();
        if(empty($connection) &&($isSaas || $walletName)){
            echo "adding connection bd!\n";
            $dbUrl = "USR_ORG_".$organizationId;
            if($walletName){
                $dbUrl = $walletName;
            }
            $dbUrlSsl = OpensslCrypt::encrypt("jdbc:oracle:thin:@".$dbUrl);
            $connectionWallet = array(
                "oracleWalletUse" => "Y",
                "name" => "MAIN_CONNECTION",
                "database" => "Oracle",
                "main" => true,
                "isActive" => false,
                "dbUrl" => null,
                "room" => null,
                "username" => null,
                "password" => null,
                "organizationId" => "",
                "service" => true,
                "languageLabel" => "Portuguese (BR)",
                "value" => "oracle",
                "dialect" => "org.hibernate.dialect.OracleDialect",
                "driverClass" => "oracle.jdbc.driver.OracleDriver",
                "label" => "Portuguese (BR)",
                "language" => "pt_br",
                "dbUrlSsl" => $dbUrlSsl,
                "usernameSsl" => "kXk13eEYQEx045ZbN7tQnQ=="
            );

            $dataSet = new DataSet('connection', array($connectionWallet));
            $orgManager->persist($dataSet);
            echo "added connection bd! $dbUrl $dbUrlSsl\n";
        }else{
            echo "Not added connection bd!\n";
        }

    }
}